#ifndef EVENTEDITOR_H
#define EVENTEDITOR_H

#include <QAction>
#include <QBuffer>
#include <QComboBox>
#include <QLineEdit>
#include <QToolButton>

#include "eventeditor/eventsetupwidget.h"
#include "filelineedit.h"
#include "eventeditor/scorinatewidget.h"
#include "eventeditor/startlistwidget.h"

class XkorEventEditor : public QWidget
{
	Q_OBJECT
	
	private:
		QLineEdit * eventName;
		XkorFileLineEdit * eventConfig;
		QToolButton * refreshConfig, * scorinate, * exportResultsButton;
		QAction * refreshAction, * scorinateAction, * exportResultsAction;
		QComboBox * sport;
		QComboBox * outputFormat;
		QString currentFileName;
		XkorEventSetupWidget * aw;
		XkorStartListWidget * slw;
		XkorScorinateWidget * scorw;
		bool fileIsModified;
		QStringList outputFormatNames, outputFormatCodes;
		
		void disableSports(QString reason);
		void getStartLists();
		bool okayToLoad();
		void writeFile(QIODevice * dev);
	public:
		XkorEventEditor(QWidget * parent = 0);
		QBuffer * getFile();
	signals:
		void fileChanged(QString filename); // loaded a new file
		void fileEdited(bool isModified); // changed the current file
	public slots:
		void openFile();
		void saveFile(QString filename = QString());
		void saveFileAs();
		void updateSports();
	private slots:
		void exportResults();
		void setModified(bool isModified = true);
		void scorinateEvent();
		void updateAthletes();
		void updateAvailableStartLists();
};

#endif
