#ifndef EVENTSETUPWIDGET_H
#define EVENTSETUPWIDGET_H

#include <QAction>
#include <QToolButton>
#include <QStringList>
#include <vector>

#include "abstracttreewidget.h"
#include "athlete.h"
#include "group.h"
#include "startlist.h"

class XkorEventSetupWidget : public XkorAbstractTreeWidget
{
	Q_OBJECT
	
	protected:
		QAction * insertGroupAction, * insertAthleteAction, * insertAllAction, * moveToTopAction, * moveToBottomAction;
		QToolButton * insertGroupButton, * insertAthleteButton, * insertAllButton, * moveToTopButton, * moveToBottomButton;
		QStringList availableAthleteNames, availableAthletes;
		std::vector<XkorStartList> slvec;
		
		XkorAthlete getAthleteBySN(QString sn) throw(XkorSearchFailedException);
		void getStartLists();
		
		virtual QTreeWidgetItem * createAthlete(QTreeWidgetItem * parent);
		virtual void initAthlete(QTreeWidgetItem * athlete) { initAthlete(athlete, QString()); }
		virtual void initAthlete(QTreeWidgetItem * athlete, QString shortName);
		virtual void initItem(QTreeWidgetItem * group) { initItem(group, QString()); } // initItem is used for groups
		virtual void initItem(QTreeWidgetItem * group, QString groupName);
		virtual void setupLayout();
		
	protected slots:
		void insertAll();
		void insertAthlete();
		void moveItemToBottom();
		void moveItemToTop();
		virtual void updateButtons();
	public:
		XkorEventSetupWidget();
		void clearAvailableStartLists();
		std::vector<XkorGroup> groups();
		void setAvailableAthleteNames(QStringList & l);
		void setAvailableAthletes(QStringList & l);
		void setAvailableStartLists(QStringList & l);
		void setGroups(std::vector<XkorGroup> g);
		void setStartLists(std::vector<XkorStartList> & l);
	signals:
		void slChanged();
	public slots:
		void updateAvailableAthletes();
};

#endif
