#ifndef ICEHOCKEY_H
#define ICEHOCKEY_H

#include "outputformats/outputformatbase.h"

template <bool overtime>
struct XkorOutputIceHockey : XkorAbstractOutputFormat
{
	public:
		virtual QString output(XkorResults * r, XkorSport & s)
		{
			QString rval;
			if(s.countValues() < requiredValues())
				return "Sport does not support this output format";
			
			for(vector<XkorResultsGroup>::iterator i = r->groups.begin(); i != r->groups.end(); ++i)
			{
				rval += "\n" + i->name + "\n";
				int acc = 0;
				for(vector<XkorAthlete>::iterator j = i->athletes.begin(); j != i->athletes.end(); ++j)
				{
					if(acc & 1) // if we’re on the second team in a match
					{
						vector<XkorAthlete>::iterator prev = j;
						--prev;
						double score1 = s.h2hScore(0, prev->skill, prev->style, prev->rpBonus, j->skill, j->style, j->rpBonus);
						double score2 = s.h2hScore(0, j->skill, j->style, j->rpBonus, prev->skill, prev->style, prev->rpBonus);
						rval += prev->name + " " + formatScore(score1, score2, s) + " " + j->name;
						if(overtime && score1 == score2)
						{
							double isOTgoal = s.individualScore(1, prev->skill, prev->style, prev->rpBonus);
							if(isOTgoal == 1)
							{
								if(s.h2hScore(2, prev->skill, prev->style, prev->rpBonus, j->skill, j->style, j->rpBonus) > s.h2hScore(2, j->skill, j->style, j->rpBonus, prev->skill, prev->style, prev->rpBonus))
									rval += " (" + formatScore(score1 + 1, score2, s) + " OT)";
								else
									rval += " (" + formatScore(score1, score2 + 1, s) + " OT)";
							}
							else
							{
								double psScore1 = 0, psScore2 = 0;
								for(int count = 0; abs(psScore1 - psScore2) <= 5 - count; ++count) // simulate PKs
								{
									if(s.h2hScore(3, prev->skill, prev->style, prev->rpBonus, j->skill, j->style, j->rpBonus) == 1)
										++psScore1;
									if(s.h2hScore(3, j->skill, j->style, j->rpBonus, prev->skill, prev->style, prev->rpBonus) == 1)
										++psScore2;
									if(count >= 5)
										--count;
								}
								rval += " (" + formatScore(score1, score2, s) + " OT; " + formatScore(psScore1, psScore2, s) + " PS)";
							}
						}
						rval += "\n";
					}
					++acc;
				}
			}
			return rval;
		}
	
	protected:
		virtual QString individualResult(XkorSport &, double, double, double) { return QString(); } // unused
		
		virtual uint requiredValues() { return 4; }
		
		QString formatScore(double score1, double score2, XkorSport & s)
		{
			return QString::number(score1, 'f', s.displayDigits(0)) + "–" + QString::number(score2, 'f', s.displayDigits(0));
		}
};

#endif
