#ifndef OUTPUTFORMATBASE_H
#define OUTPUTFORMATBASE_H

#include <cmath>

struct XkorAbstractOutputFormat
{
	public:
		virtual QString output(XkorResults * r, XkorSport & s)
		{
			QString rval;
			if(s.countValues() < requiredValues())
				return "Sport does not support this output format";
			
			for(vector<XkorResultsGroup>::iterator i = r->groups.begin(); i != r->groups.end(); ++i)
			{
				rval += "\n" + i->name + "\n";
				for(vector<XkorAthlete>::iterator j = i->athletes.begin(); j != i->athletes.end(); ++j)
				{
					rval += athleteName(j->name, j->nation).leftJustified(48);
					rval += individualResult(s, j->skill, j->style, j->rpBonus).rightJustified(10);
					rval += "\n";
				}
			}
			return rval;
		}
		
		virtual ~XkorAbstractOutputFormat() {;;;}
	
	protected:
		virtual QString athleteName(QString name, QString nation)
		{
			return name + " (" + nation + ")";
		}
		
		virtual QString individualResult(XkorSport & s, double skill, double style, double bonus) = 0;
		
		virtual uint requiredValues() = 0;
		
		virtual QString timeFormat(double n, uint displayDigits)
		{
			QString rval;
			if(n > 3600) // hours
			{
				rval += QString::number((int)(n / 3600));
				rval += ":";
				if((int)n % 3600 < 600) // need a leading zero for minutes
					rval += "0";
			}
			if(n > 60) // minutes
			{
				rval += QString::number((int)(n / 60) % 60);
				rval += ":";
				if((int)n % 60 < 10) // need a leading zero for seconds
					rval += "0";
			}
			rval += QString::number((int)n % 60);
			if(displayDigits > 0)
			{
				rval += ".";
				int remainder = (int)((n - (int)n) * pow(10.0, (int)displayDigits)); // e.g., if time is 10.06, this will be 6
				for(uint i = 1; i < displayDigits; ++i) // leading zeros
				{
					if(remainder < pow(10.0, (int)i))
						rval += "0";
				}
				rval += QString::number((int)((n - (int)n) * pow(10.0, (int)displayDigits)));
			}
			return rval;
		}
};

#endif
