#include "startlist.h"

#include <algorithm>
#include "xml/xmlstartlisthandler.h"

using namespace std;

XkorStartList::XkorStartList(const QString & filename)
{
	// set up the input source
	// TODO: error handling
	QFile * f = new QFile(filename);
	QXmlInputSource * in = new QXmlInputSource(f);
	
	// parse the XML file
	QXmlSimpleReader r;
	XkorXmlStartListHandler * h = new XkorXmlStartListHandler;
	r.setContentHandler(h);
	r.parse(in, false);
	
	this->name = h->getName();
	this->athletes = h->getAthletes();
	
	delete h;
	delete in;
	delete f;
}

QString XkorStartList::getName()
{
	return this->name;
}

void XkorStartList::setName(const QString & new_name)
{
	this->name = new_name;
}

std::set<XkorAthlete> XkorStartList::getAthletes()
{
	return this->athletes;
}

void XkorStartList::addAthlete(const XkorAthlete & a)
{
	this->athletes.insert(a);
}

XkorAthlete XkorStartList::getAthleteBySN(const QString & name) throw(XkorSearchFailedException)
{
	set<XkorAthlete>::iterator val;
	val = lower_bound(athletes.begin(), athletes.end(), XkorAthlete(name));
	if(val != athletes.end() && val->shortname == name)
		return *val;
	else
	{
		std::string err = "No athlete with shortname ";
		err += name.toStdString();
		err += " in XkorStartList::getAthleteBySN(const QString &)";
		throw XkorSearchFailedException(err);
	}
}
