#include "tablegenerator/tablecolumnswidget.h"
#include "tablegenerator/tablecolumndelegate.h"

#include <QGridLayout>
#include <QHeaderView>
#include <QToolButton>

XkorTableColumnsWidget::XkorTableColumnsWidget()
{
	columnTypes << "draws" << "played" << "goalAverage" << "goalDifference" << "goalsAgainst"  << "goalsFor" << "losses" << "name" << "points" << "position" << "resultsGrid" << "winPercent" << "wins";
	columnNames << tr("Draws") << tr("Games played") << tr("Goal average") << tr("Goal difference") << tr("Goals against") << tr("Goals for") << tr("Losses") << tr("Name") << tr("Points") << tr("Position") << tr("Results grid") << tr("Win percentage") << tr("Wins");
	treeWidget->setColumnCount(3);
	treeWidget->setItemDelegate(new XkorTableColumnDelegate(columnTypes, columnNames));
	treeWidget->setHeaderLabels(QStringList() << tr("Column") << tr("Heading") << tr("Width"));
	treeWidget->header()->setResizeMode(0, QHeaderView::Stretch);
	treeWidget->header()->setResizeMode(1, QHeaderView::Stretch);
	treeWidget->header()->setResizeMode(2, QHeaderView::ResizeToContents);
	treeWidget->header()->setStretchLastSection(false);
	
	setupLayout();
}

std::vector<XkorTableColumn> XkorTableColumnsWidget::columns()
{
	std::vector<XkorTableColumn> rval;
	for(QTreeWidgetItemIterator i(treeWidget); *i; ++i)
		rval.push_back(XkorTableColumn((*i)->data(0, 1093).toString(), (*i)->text(1), (*i)->text(2).toUInt()));
	return rval;
}

void XkorTableColumnsWidget::initItem(QTreeWidgetItem * item, QString columnType, QString heading, uint width)
{
	item->setData(0, 1093, columnType);
	item->setText(1, heading);
	item->setText(2, QString::number(width));
		
	// get the name of the column type
	int n = columnTypes.indexOf(columnType);
	if(n >= 0 && n < columnNames.size())
		item->setText(0, columnNames[n]);
	else
		item->setText(0, tr("<unknown column type %1>").arg(columnType));
}

void XkorTableColumnsWidget::setColumns(std::vector<XkorTableColumn> c)
{
	isInUse = true;
	clear();
	for(std::vector<XkorTableColumn>::iterator i = c.begin(); i != c.end(); ++i)
		initItem(createItem(), i->columnType, i->heading, i->width);
	isInUse = false;
	emit listChanged();
}
