#include "tablerow.h"

double XkorTableRow::awayGoals() const
{
	double rval = 0;
	for(std::vector<XkorTableMatch>::const_iterator i = h2hResults.begin(); i != h2hResults.end(); ++i)
	{
		if(i->team2 == n)
			rval += i->goalsFor(n);
	}
	return rval;
}

double XkorTableRow::draws() const
{
	return d;
}

double XkorTableRow::goalsAgainst() const
{
	return ga;
}

double XkorTableRow::goalDifference() const
{
	return gf - ga;
}

double XkorTableRow::goalsFor() const
{
	return gf;
}

double XkorTableRow::h2hAwayGoals(QStringList & teams) const
{
	double rval = 0;
	for(std::vector<XkorTableMatch>::const_iterator i = h2hResults.begin(); i != h2hResults.end(); ++i)
	{
		if(teams.indexOf(i->team1) != -1 && i->team2 == n)
			rval += i->goalsFor(n);
	}
	return rval;
}

double XkorTableRow::h2hGoalDifference(QStringList & teams) const
{
	double rval = 0;
	for(std::vector<XkorTableMatch>::const_iterator i = h2hResults.begin(); i != h2hResults.end(); ++i)
	{
		if(teams.indexOf(i->opponent(n)) != -1) // if the opponent in this match is one of the teams we’re looking for
			rval += i->goalDifference(n);
	}
	return rval;
}

double XkorTableRow::h2hGoalsFor(QStringList & teams) const
{
	double rval = 0;
	for(std::vector<XkorTableMatch>::const_iterator i = h2hResults.begin(); i != h2hResults.end(); ++i)
	{
		if(teams.indexOf(i->opponent(n)) != -1) // if the opponent in this match is one of the teams we’re looking for
			rval += i->goalsFor(n);
	}
	return rval;
}

double XkorTableRow::h2hPoints(QStringList & teams) const
{
	double rval = 0;
	for(std::vector<XkorTableMatch>::const_iterator i = h2hResults.begin(); i != h2hResults.end(); ++i)
	{
		if(teams.indexOf(i->opponent(n)) != -1) // if the opponent in this match is one of the teams we’re looking for
			rval += i->points(n);
	}
	return rval;
}

double XkorTableRow::h2hWins(QStringList & teams) const
{
	double rval = 0;
	for(std::vector<XkorTableMatch>::const_iterator i = h2hResults.begin(); i != h2hResults.end(); ++i)
	{
		if(teams.indexOf(i->opponent(n)) != -1) // if the opponent in this match is one of the teams we’re looking for
			rval += (i->points(n) == 3 ? 1 : 0);
	}
	return rval;
}

void XkorTableRow::insertMatch(QString & t2name, double t1score, double t2score, bool home)
{
	if(home)
		h2hResults.push_back(XkorTableMatch(n, t2name, t1score, t2score));
	else
		h2hResults.push_back(XkorTableMatch(t2name, n, t2score, t1score));
	if(t1score > t2score)
		++w;
	else if(t1score == t2score)
		++d;
	else
		++l;
	gf += t1score;
	ga += t2score;
}

double XkorTableRow::losses() const
{
	return l;
}

QString XkorTableRow::name() const
{
	return n;
}

double XkorTableRow::played() const
{
	return w + d + l;
}

double XkorTableRow::points() const
{
	return w * 3 + d;
}

std::pair<double, double> XkorTableRow::result(QString opponent) const
{
	std::pair<double, double> rval = std::make_pair(-1, -1);
	for(std::vector<XkorTableMatch>::const_iterator i = h2hResults.begin(); i != h2hResults.end(); ++i)
		if(i->team1 == n && i->team2 == opponent)
			rval = std::make_pair(i->score1, i->score2);
	return rval;
}

double XkorTableRow::wins() const
{
	return w;
}
