#include "xml_index.h"

#include <QDir>
#include <QDirIterator>
#include <QFile>
#include "xml/xmlbasehandler.h"

using namespace std;

void XkorXmlIndex::insert(QString filename)
{
	// set up the input source
	QFile * f = new QFile(filename);
	QXmlInputSource * in = new QXmlInputSource(f);
	
	// parse the XML file
	QXmlSimpleReader r;
	XkorXmlBaseHandler * h = new XkorXmlBaseHandler;
	r.setContentHandler(h);
	r.parse(in, false);
	
	// insert into the map
	index[h->getName()] = filename;
	
	delete h;
	delete f;
	delete in;
}

void XkorXmlIndex::traverse(QString dir) throw(XkorFileNotFoundException)
{
	// initialize a QDir to the desired directory
	QDir d;
	d.setPath(dir);
	if(!d.exists())
	{
		string err = "Directory ‘";
		err += d.path().toStdString();
		err += "’ not found by XkorXmlIndex::traverse(QString)";
		throw XkorFileNotFoundException(err, QObject::tr("sport directory"), d.path());
	}
	QDirIterator i(d, QDirIterator::Subdirectories);
	// iterate
	while(i.hasNext())
	{
		QString f = i.next();
		if(i.fileInfo().isFile())
			this->insert(f);
	}
}

QString XkorXmlIndex::lookup(QString name) throw(XkorSearchFailedException)
{
	map<QString, QString>::iterator i = index.find(name);
	if(i == index.end())
	{
		string err = "Unknown object ‘";
		err += name.toStdString();
		err += "’ in XkorXmlIndex::lookup(QString)";
		throw XkorSearchFailedException(err);
	}
	return (i->second);
}

std::map<QString, QString> XkorXmlIndex::getAllFiles()
{
	return index;
}
